#!/bin/sh

K_VERSION_EXPECTED="2.6.35.7"
DIR_K_MODULES="/lib/modules/$K_VERSION_EXPECTED/kernel/drivers/net"

ETH_ALIAS_OLD="r8169"
ETH_ALIAS_NEW="r8168"  

K_MODULE_NEW="r8168.ko"
K_MODULE_OLD="r8169.ko"

K_MODULE_OLD_PATH="$DIR_K_MODULES/$K_MODULE_OLD"
K_MODULE_NEW_PATH="$DIR_K_MODULES/$K_MODULE_NEW"

FILE_MODPROBE_CONF="/etc/modprobe.conf"
 

FILE_MESSAGES="/var/log/messages"
FILE_MODPROBE_CONF="/etc/modprobe.conf"


DIR_BACKUP="/root/r8169-backup"

################################################

result()
{
	if [ $? -ne 0 ]; then
		echo "ERROR"
		echo
		echo "$0 failed"
		echo
		exit 1
	fi
}


check_kernel_version()
{
	local current_kver=`uname -r`
	echo "Current kernel version is $current_kver"
	if [[ "$current_kver" == "$K_VERSION_EXPECTED" ]]; then
		echo "Proceeding upgrade to kernel $K_VERSION_EXPECTED"
	else
		echo "Wrong kernel version: expected($K_VERSION_EXPECTED) found($current_kver)"
		exit 1
	fi
}

catv()
{
	local file_to_show=$1

	if [ ! -f $file_to_show ]; then
		echo "WARNING! catv() File $file_to_show not found"
		return 1
	fi
	
	echo
	echo
	echo "--- $file_to_show content ---"
	cat $file_to_show
	echo
	echo "--- $file_to_show content ---"
	echo
	echo
}  

cpv()
{
	local src=$1
	local dst=$2
	
	echo
	if [ -z "$src" ]; then
		echo "ERROR: cpv(): src argument is empty"
		exit 1
	fi
	
	if [ -z "$dst" ]; then
		echo "ERROR: cpv(): dst argument is empty"
		exit 1
	fi
	
	echo "cpv(): Copying $src to $dst"
	cp -afv $CP_OPTIONS $src $dst
	result
}


dir_create()
{
	local dir=$1
	
	if [ -z "$dir" ]; then
		echo "ERROR: dir_create() argument 1 is empty"
		exit 1
	fi
	
	echo "dir_create($dir) - creating directory"
	mkdir -pv $dir
	result
}

eth_alias_setup()
{
	local alias_old=$1
	local alias_new=$2
	
	catv $FILE_MODPROBE_CONF
	
	echo
	echo -n "Searching $FILE_MODPROBE_CONF for \"$alias_old\" ... "
	grep "^alias eth0 $alias_old" $FILE_MODPROBE_CONF > /dev/null
	if [ $? -eq 0 ]; then
		echo "found"
		echo "Replacing $alias_old with $alias_new"
		sed -i -e "s/$alias_old/$alias_new/" $FILE_MODPROBE_CONF
		result
		
		echo
		echo "Setting access mode for $FILE_MODPROBE_CONF"
		chmod a+r $FILE_MODPROBE_CONF
		echo
		
		catv $FILE_MODPROBE_CONF
	else
		echo "not found"
		echo
		echo "This host doesn't use \"$eth_alias_old\""
		echo "Terminating work"
		echo
		exit 1
	fi
} 

backup_files()
{
	echo -n "Checking $DIR_BACKUP ... "
	if [ -d $DIR_BACKUP ]; then
		echo "found"
		echo "Nothing to do"
	else
		echo "not found"
		dir_create $DIR_BACKUP
		
		cpv $K_MODULE_OLD_PATH $DIR_BACKUP
		cpv $FILE_MODPROBE_CONF $DIR_BACKUP
	fi
}

####################################################################
# Script code

echo "==== $(date) $0 begin =============================="

check_kernel_version

backup_files

echo
echo "Installing kernel module $K_MODULE_NEW to $K_MODULE_NEW_PATH"
cpv $K_MODULE_NEW $K_MODULE_NEW_PATH

echo "Setting owner for $K_MODULE_NEW_PATH"
chown -v root:root $K_MODULE_NEW_PATH
result

echo "Setting access mode for $K_MODULE_NEW_PATH"
chmod -v 644 $K_MODULE_NEW_PATH
result

echo "Updating kernel modules cache"
depmod -a
result
echo

eth_alias_setup $ETH_ALIAS_OLD $ETH_ALIAS_NEW


echo "Rebooting system"
reboot

echo "==== $(date) $0 end =============================="

exit 0